# Parses command line arguments
# Formatspec is defined as follows:
#     <name>:<type>[:<default>],<name>:<type>[:<default>], ...
# currently 4 types are supported:
#     boolean(b), string(s), integer(i), float(f)

class argument_container:

    def __init__(self):
        self.name = ''
        self.type = ''
        self.mandatory = True
        self.default = ''
        self.index = 0
        self.present = False
        self.value = ''

    def verify_and_convert_parameters(self):
        if self.type == "b":
            self.verify_and_convert_boolean_parameters()
            return
        if self.type == "s":
            self.verify_and_convert_string_parameters()
            return
        if self.type == "i":
            self.verify_and_convert_integer_parameters()
            return
        if self.type == "f":
            self.verify_and_convert_float_parameters()
            return

        raise TypeError('Unrecognized parameter type: ' + self.type)

    def verify_and_convert_boolean_parameters(self):
        self.value = self.present

    def verify_and_convert_string_parameters(self):
        self.raise_error_if_mandatory_but_not_present()

    def verify_and_convert_integer_parameters(self):
        self.raise_error_if_mandatory_but_not_present()
        self.value = int(self.value)

    def verify_and_convert_float_parameters(self):
        self.raise_error_if_mandatory_but_not_present()
        self.value = float(self.value)

    def raise_error_if_mandatory_but_not_present(self):
        if self.mandatory:
            if not self.present:
                raise ValueError('Parameter ' + self.name + ' mandatory but not present.')        

class arg:

    def __init__(self, fmt, argv):
        self.arguments = []
        self.arg_cont = argument_container()
        
        self.parse_formatspec(str(fmt), argv)

    def parse_formatspec(self, fmt, argv):
        fmt = fmt.split(',')

        for spec in fmt:
            self.parse_single_parameter(spec, argv)
        
        if len(argv) > 1:
            raise ValueError('Unrecognized parameters: ' + str(argv[1:]))

    def parse_single_parameter(self, fmt, argv):
        self.arg_cont = argument_container()

        self.parse_formatspec_values_into_container(fmt)
        self.find_parameter_in_parameterlist_and_update_container(argv)
        self.get_parameter_value_and_update_container(argv)

        self.arg_cont.verify_and_convert_parameters()
        self.arguments.append(self.arg_cont)

    def parse_formatspec_values_into_container(self, fmt):
        fmt = fmt.split(':')
        self.arg_cont.name = fmt[0]
        self.arg_cont.type = fmt[1]
        self.arg_cont.mandatory = True
        self.arg_cont.default = ''
        if len(fmt) > 2:
            self.arg_cont.mandatory = False
            self.arg_cont.default = fmt[2]

    def find_parameter_in_parameterlist_and_update_container(self, argv):
        try:
            self.arg_cont.index = argv.index('-' + self.arg_cont.name)
            self.arg_cont.present = True
            del argv[self.arg_cont.index]
        except ValueError:
            self.arg_cont.present = False

    def get_parameter_value_and_update_container(self, argv):
        self.arg_cont.value = self.arg_cont.default
        if self.arg_cont.present:
            if len(argv) >= self.arg_cont.index + 1:
                if argv[self.arg_cont.index][0] != '-':
                    self.arg_cont.value = argv[self.arg_cont.index]
                    del argv[self.arg_cont.index]

    def get_parameter_by_name(self, name):
        for arg_cont in self.arguments:
            if arg_cont.name == name:
                return(arg_cont.value)
        raise ValueError('No argument with name ' + name + ' found.')

# May rise the following Errors:
# 1: ValueError: parameter mandatory but not present
# 2: ValueError: parameter of wrong type
# 3: ValueError: unrecognized parameters
# 4: ValueError: call to get_value_by_name with unrecognized name
# 5: TypeError:  unrecognized type definition in format spec
